/*

 petscii.c - A mostly working PCLink server for IDEDOS 0.9x

 Written by
  Kajtar Zsolt <soci@c64.rulez.org>

    This file is part of IDEDOS the IDE64 disk operating system
    See README for copyright notice.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "petscii.h"

Petscii unicode_to_petscii(unsigned int w) {
    static const Petscii latinconv1[] = {
        0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xc1, 0xa4, 0xc3, 0xc5, 0xc5, 0xc5, 0xc5, 0xc9,
        0xc9, 0xc9, 0xc9, 0xa4, 0xce, 0xcf, 0xcf, 0xcf, 0xcf, 0xcf, 0xa4, 0xa4, 0xd5,
        0xd5, 0xd5, 0xd5, 0xd9, 0xa4, 0xa4, 0x41, 0x41, 0x41, 0x41, 0x41, 0x41, 0xa4,
        0x43, 0x45, 0x45, 0x45, 0x45, 0x49, 0x49, 0x49, 0x49, 0xa4, 0x4e, 0x4f, 0x4f,
        0x4f, 0x4f, 0x4f, 0xa4, 0xa4, 0x55, 0x55, 0x55, 0x55, 0x59, 0xa4, 0x59, 0xc1,
        0x41, 0xc1, 0x41, 0xc1, 0x41, 0xc3, 0x43, 0xc3, 0x43, 0xc3, 0x43, 0xc3, 0x43,
        0xc4, 0x44, 0xa4, 0xa4, 0xc5, 0x45, 0xc5, 0x45, 0xc5, 0x45, 0xc5, 0x45, 0xc5,
        0x45, 0xc7, 0x47, 0xc7, 0x47, 0xc7, 0x47, 0xc7, 0x47, 0xc8, 0x48, 0xa4, 0xa4,
        0xc9, 0x49, 0xc9, 0x49, 0xc9, 0x49, 0xc9, 0x49, 0xc9, 0xa4, 0xa4, 0xa4, 0xca,
        0x4a, 0xcb, 0x4b, 0xa4, 0xcc, 0x4c, 0xcc, 0x4c, 0xcc, 0x4c, 0xa4, 0xa4, 0xa4,
        0xa4, 0xce, 0x4e, 0xce, 0x4e, 0xce, 0x4e, 0xa4, 0xa4, 0xa4, 0xcf, 0x4f, 0xcf,
        0x4f, 0xcf, 0x4f, 0xa4, 0xa4, 0xd2, 0x52, 0xd2, 0x52, 0xd2, 0x52, 0xd3, 0x53,
        0xd3, 0x53, 0xd3, 0x53, 0xd3, 0x53, 0xd4, 0x54, 0xd4, 0x54, 0xa4, 0xa4, 0xd5,
        0x55, 0xd5, 0x55, 0xd5, 0x55, 0xd5, 0x55, 0xd5, 0x55, 0xd5, 0x55, 0xd7, 0x57,
        0xd9, 0x59, 0xd9, 0xda, 0x5a, 0xda, 0x5a, 0xda, 0x5a, 0xa4, 0xa4, 0xa4, 0xa4,
        0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4,
        0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4,
        0xa4, 0xa4, 0xa4, 0xcf, 0x4f, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4,
        0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xd5, 0x55, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4,
        0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4,
        0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xc1, 0x41, 0xc9, 0x49,
        0xcf, 0x4f, 0xd5, 0x55, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4,
        0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xc7, 0x47, 0xcb, 0x4b, 0xcf,
        0x4f, 0xa4, 0xa4, 0xa4, 0xa4, 0x4a, 0xa4, 0xa4, 0xa4, 0xc7, 0x47, 0xa4, 0xa4,
        0xce, 0x4e, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xc1, 0x41, 0xc1, 0x41, 0xc5,
        0x45, 0xc5, 0x45, 0xc9, 0x49, 0xc9, 0x49, 0xcf, 0x4f, 0xcf, 0x4f, 0xd2, 0x52,
        0xd2, 0x52, 0xd5, 0x55, 0xd5, 0x55, 0xd3, 0x53, 0xd4, 0x54, 0xa4, 0xa4, 0xc8,
        0x48, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xc1, 0x41, 0xc5, 0x45, 0xa4, 0xa4,
        0xa4, 0xa4, 0xcf, 0x4f, 0xa4, 0xa4, 0xd9, 0x59
    };
    static const Petscii latinconv2[] = {
        0xc1, 0x41, 0xc2, 0x42, 0xc2, 0x42, 0xc2, 0x42, 0xa4, 0xa4, 0xc4, 0x44, 0xc4,
        0x44, 0xc4, 0x44, 0xc4, 0x44, 0xc4, 0x44, 0xa4, 0xa4, 0xa4, 0xa4, 0xc5, 0x45,
        0xc5, 0x45, 0xa4, 0xa4, 0xc6, 0x46, 0xc7, 0x47, 0xc8, 0x48, 0xc8, 0x48, 0xc8,
        0x48, 0xc8, 0x48, 0xc8, 0x48, 0xc9, 0x49, 0xa4, 0xa4, 0xcb, 0x4b, 0xcb, 0x4b,
        0xcb, 0x4b, 0xcc, 0x4c, 0xa4, 0xa4, 0xcc, 0x4c, 0xcc, 0x4c, 0xcd, 0x4d, 0xcd,
        0x4d, 0xcd, 0x4d, 0xce, 0x4e, 0xce, 0x4e, 0xce, 0x4e, 0xce, 0x4e, 0xa4, 0xa4,
        0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xd0, 0x50, 0xd0, 0x50, 0xd2, 0x52, 0xd2,
        0x52, 0xa4, 0xa4, 0xd2, 0x52, 0xd3, 0x53, 0xd3, 0x53, 0xa4, 0xa4, 0xa4, 0xa4,
        0xa4, 0xa4, 0xd4, 0x54, 0xd4, 0x54, 0xd4, 0x54, 0xd4, 0x54, 0xd5, 0x55, 0xd5,
        0x55, 0xd5, 0x55, 0xa4, 0xa4, 0xa4, 0xa4, 0xd6, 0x56, 0xd6, 0x56, 0xd7, 0x57,
        0xd7, 0x57, 0xd7, 0x57, 0xd7, 0x57, 0xd7, 0x57, 0xd8, 0x58, 0xd8, 0x58, 0xd9,
        0x59, 0xda, 0x5a, 0xda, 0x5a, 0xda, 0x5a, 0x48, 0x54, 0x57, 0x59, 0xa4, 0xa4,
        0xa4, 0xa4, 0xa4, 0xa4, 0xc1, 0x41, 0xc1, 0x41, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4,
        0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4,
        0xa4, 0xa4, 0xc5, 0x45, 0xc5, 0x45, 0xc5, 0x45, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4,
        0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xc9, 0x49, 0xc9, 0x49, 0xcf, 0x4f, 0xcf, 0x4f,
        0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4,
        0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xd5, 0x55, 0xd5, 0x55, 0xa4, 0xa4,
        0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xa4, 0xd9, 0x59, 0xd9, 0x59, 0xd9,
        0x59, 0xd9, 0x59
    };

    if (w >= 0x20 && w <= 0x40) return (Petscii)w;
    if (w >= 0x41 && w <= 0x5a) return (Petscii)(w + 0x80);
    if (w >= 0x61 && w <= 0x7a) return (Petscii)(w - 0x20);
    if (w >= 0xe100 && w <= 0xe1ff) return (Petscii)(w - 0xe100);
    switch (w) {
    case 0x2215: return 0x2f;
    case 0x5b:   return 0x5b;
    case 0xa3:   return 0x5c;
    case 0x5d:   return 0x5d;
    case 0x2191: return 0x5e;
    case 0x2190: return 0x5f;
    case 0xa0:   return 0xa0;
    case 0x258c: return 0xa1;
    case 0x2584: return 0xa2;
    case 0x2594: return 0xa3;
    case 0x5f:   return 0xa4;
    case 0x258f: return 0xa5;
    case 0x2592: return 0xa6;
    case 0x2595: return 0xa7;
    case 0x1fb8f:return 0xa8;
    case 0x1fb99:return 0xa9;
    case 0x1fb87:return 0xaa;
    case 0x251c: return 0xab;
    case 0x2597: return 0xac;
    case 0x2514: return 0xad;
    case 0x2510: return 0xae;
    case 0x2582: return 0xaf;
    case 0x250c: return 0xb0;
    case 0x2534: return 0xb1;
    case 0x252c: return 0xb2;
    case 0x2524: return 0xb3;
    case 0x258e: return 0xb4;
    case 0x258d: return 0xb5;
    case 0x1fb88:return 0xb6;
    case 0x1fb82:return 0xb7;
    case 0x1fb83:return 0xb8;
    case 0x2583: return 0xb9;
    case 0x2713: return 0xba;
    case 0x2596: return 0xbb;
    case 0x259d: return 0xbc;
    case 0x2518: return 0xbd;
    case 0x2598: return 0xbe;
    case 0x259a: return 0xbf;
    case 0x2500: return 0xc0;
    case 0x253c: return 0xdb;
    case 0x1fb8c:return 0xdc;
    case 0x2502: return 0xdd;
    case 0x1fb98:return 0xdf;
    case 0x03c0: return 0xff;
    default: 
        if (w >= 0xc0 && w < 0xc0 + sizeof latinconv1) {
            return latinconv1[w - 0xc0];
        }
        if (w >= 0x1e00 && w < 0x1e00 + sizeof latinconv2) {
            return latinconv2[w - 0x1e00];
        }
        return 0xa4;
    }
}

unsigned int petscii_to_unicode(Petscii c) {
    if (c >= 0x20 && c <= 0x40) return (c != 0x2f) ? c : 0x2215;
    if (c >= 0x41 && c <= 0x5a) return c + 0x20u;
    if (c >= 0xc1 && c <= 0xda) return c - 0x80u;
    switch (c) {
    case 0x5b: return 0x5b;
    case 0x5c: return 0xa3;
    case 0x5d: return 0x5d;
    case 0x5e: return 0x2191;
    case 0x5f: return 0x2190;
    case 0xa0: return 0xa0;
    case 0xa1: return 0x258c;
    case 0xa2: return 0x2584;
    case 0xa3: return 0x2594;
    case 0xa4: return 0x5f;
    case 0xa5: return 0x258f;
    case 0xa6: return 0x2592;
    case 0xa7: return 0x2595;
    case 0xa8: return 0x1fb8f;
    case 0xa9: return 0x1fb99;
    case 0xaa: return 0x1fb87;
    case 0xab: return 0x251c;
    case 0xac: return 0x2597;
    case 0xad: return 0x2514;
    case 0xae: return 0x2510;
    case 0xaf: return 0x2582;
    case 0xb0: return 0x250c;
    case 0xb1: return 0x2534;
    case 0xb2: return 0x252c;
    case 0xb3: return 0x2524;
    case 0xb4: return 0x258e;
    case 0xb5: return 0x258d;
    case 0xb6: return 0x1fb88;
    case 0xb7: return 0x1fb82;
    case 0xb8: return 0x1fb83;
    case 0xb9: return 0x2583;
    case 0xba: return 0x2713;
    case 0xbb: return 0x2596;
    case 0xbc: return 0x259d;
    case 0xbd: return 0x2518;
    case 0xbe: return 0x2598;
    case 0xbf: return 0x259a;
    case 0xc0: return 0x2500;
    case 0xdb: return 0x253c;
    case 0xdc: return 0x1fb8c;
    case 0xdd: return 0x2502;
    case 0xdf: return 0x1fb98;
    case 0xff: return 0x03c0;
    default:   return 0xe100u + c;
    }
}

