/*

 ws2.c

 Written by
  Kajtar Zsolt <soci@c64.rulez.org>

    This file is part of IDEDOS the IDE64 disk operating system
    See README for copyright notice.

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/
#include "ws2.h"
#ifdef _WS2_H
#include "log.h"
#include "message.h"

lpWSAStartup WSAStartup;
lpgethostbyname gethostbyname;
lpWSAGetLastError WSAGetLastError;
lphtonl htonl;
lphtons htons;
lpsocket socket;
lpWSACreateEvent WSACreateEvent;
lpconnect connect;
lpclosesocket closesocket;
lpsetsockopt setsockopt;
lpWSAResetEvent WSAResetEvent;
lpWSARecv WSARecv;
lpWSAWaitForMultipleEvents WSAWaitForMultipleEvents;
lpWSAGetOverlappedResult WSAGetOverlappedResult;
lpshutdown shutdown;
lpWSAIoctl WSAIoctl;
lpbind bind;
lpWSACloseEvent WSACloseEvent;
lpWSACleanup WSACleanup;
lpsendto sendto;
lprecvfrom recvfrom;
lpsend send;
lpntohs ntohs;
lpinet_addr inet_addr;

static HINSTANCE hLib;

static void *lookup(const char *name) {
    void *addr = GetProcAddress(hLib, name);
    if (addr == NULL) { 
        DWORD error = GetLastError();
        log_printf("GetProcAddress for %s failed: %s(%u)", name, errormsg(error), (unsigned int)error); 
    }
    return addr;
}

int ws2_init(void) {
    if (hLib == NULL) {
        hLib = LoadLibrary("ws2_32.dll");
        if (hLib == NULL) {
            DWORD error = GetLastError();
            log_printf("LoadLibrary of ws2_32.dll failed: %s(%u)", errormsg(error), (unsigned int)error); 
            return 0; 
        }
    }
    WSAStartup = (lpWSAStartup)lookup("WSAStartup");
    if (WSAStartup == NULL) return 0;
    gethostbyname = (lpgethostbyname)lookup("gethostbyname");
    if (gethostbyname == NULL) return 0;
    WSAGetLastError = (lpWSAGetLastError)lookup("WSAGetLastError");
    if (WSAGetLastError == NULL) return 0;
    htonl = (lphtonl)lookup("htonl");
    if (htonl == NULL) return 0;
    htons = (lphtons)lookup("htons");
    if (htons == NULL) return 0;
    socket = (lpsocket)lookup("socket");
    if (socket == NULL) return 0;
    WSACreateEvent = (lpWSACreateEvent)lookup("WSACreateEvent");
    if (WSACreateEvent == NULL) return 0;
    connect = (lpconnect)lookup("connect");
    if (connect == NULL) return 0;
    closesocket = (lpclosesocket)lookup("closesocket");
    if (closesocket == NULL) return 0;
    setsockopt = (lpsetsockopt)lookup("setsockopt");
    if (setsockopt == NULL) return 0;
    WSAResetEvent = (lpWSAResetEvent)lookup("WSAResetEvent");
    if (WSAResetEvent == NULL) return 0;
    WSARecv = (lpWSARecv)lookup("WSARecv");
    if (WSARecv == NULL) return 0;
    WSAWaitForMultipleEvents = (lpWSAWaitForMultipleEvents)lookup("WSAWaitForMultipleEvents");
    if (WSAWaitForMultipleEvents == NULL) return 0;
    WSAGetOverlappedResult = (lpWSAGetOverlappedResult)lookup("WSAGetOverlappedResult");
    if (WSAGetOverlappedResult == NULL) return 0;
    shutdown = (lpshutdown)lookup("shutdown");
    if (shutdown == NULL) return 0;
    WSAIoctl = (lpWSAIoctl)lookup("WSAIoctl");
    if (WSAIoctl == NULL) return 0;
    bind = (lpbind)lookup("bind");
    if (bind == NULL) return 0;
    WSACloseEvent = (lpWSACloseEvent)lookup("WSACloseEvent");
    if (WSACloseEvent == NULL) return 0;
    WSACleanup = (lpWSACleanup)lookup("WSACleanup");
    if (WSACleanup == NULL) return 0;
    sendto = (lpsendto)lookup("sendto");
    if (sendto == NULL) return 0;
    recvfrom = (lprecvfrom)lookup("recvfrom");
    if (recvfrom == NULL) return 0;
    send = (lpsend)lookup("send");
    if (send == NULL) return 0;
    ntohs = (lpntohs)lookup("ntohs");
    if (ntohs == NULL) return 0;
    inet_addr = (lpinet_addr)lookup("inet_addr");
    if (inet_addr == NULL) return 0;
    return 1;
}

void ws2_deinit(void) {
    if (hLib != NULL) {
        FreeLibrary(hLib);
        hLib = NULL;
    }
    WSAStartup = NULL;
    gethostbyname = NULL;
    WSAGetLastError = NULL;
    htonl = NULL;
    htons = NULL;
    socket = NULL;
    WSACreateEvent = NULL;
    connect = NULL;
    closesocket = NULL;
    setsockopt = NULL;
    WSAResetEvent = NULL;
    WSARecv = NULL;
    WSAWaitForMultipleEvents = NULL;
    WSAGetOverlappedResult = NULL;
    shutdown = NULL;
    WSAIoctl = NULL;
    bind = NULL;
    WSACloseEvent = NULL;
    WSACleanup = NULL;
    sendto = NULL;
    recvfrom = NULL;
    send = NULL;
    ntohs = NULL;
}
#endif
